#ifndef _BJPEG_H_
#define _BJPEG_H_

#include "IUndoStore.h"

enum { eThumbNever, eThumbKeepExisting, eThumbRegenerateExisting };
enum { eCompressChangesOnly, eCompressAll };
enum { eSampling1, eSampling2 };
enum { eThumbDrawWhiteBorder=1, eThumbDrawDarkBorder=2, eThumbDrawShadow=4};

struct save_params {
	save_params():
		iCompressionMode(eCompressChangesOnly),
		iQuality(75),
		iColorSampling(eSampling2),
		bRetainOriginalTime(FALSE),
		bOptimizeHuffman(TRUE),
		bSaveComments(TRUE),
		bSaveEXIF(TRUE),
		// Thumbnail
		iGenerateEXIFThumb(eThumbKeepExisting),
		iEXIFThumbSize(160),
		// Bitmap
		pScan0(NULL),
		iStride(0) {};
	int iCompressionMode;
	int iQuality;
	int iColorSampling;
	BOOL bRetainOriginalTime;
	BOOL bOptimizeHuffman;
	BOOL bSaveComments;
	BOOL bSaveEXIF;
	// Thumbnail
	int iGenerateEXIFThumb;
	int iEXIFThumbSize;
	// Bitmap
	BYTE* pScan0;
	int iStride;
};

struct recompress_statistics {
	int iMCUsTotal;
	int iDCTsTotal;
	int iMCUsChanged;
	int iDCTsChanged;
	recompress_statistics(): iMCUsTotal(0), iDCTsTotal(0), iMCUsChanged(0), iDCTsChanged(0) {}
};

struct thumbnail_info {
	USHORT iWidth;
	USHORT iHeight;
};

typedef BOOL (STDMETHODCALLTYPE *PFNPROGRESS)(int iProgress);

// {163753f0-d70f-4eda-b3dc-a68e7e75b31d}
static const GUID IID_IBJPEG =
{ 0x163753f0, 0xd70f, 0x4eda, { 0xb3, 0xdc, 0xa6, 0x8e, 0x7e, 0x75, 0xb3, 0x1d } };

interface IBJPEG : public IUnknown
{
	enum ETypeOfReadImage {eReadWholeImage, eReadExifOnly};
	virtual BOOL STDMETHODCALLTYPE ReadImage(WCHAR* pwsFileName, ETypeOfReadImage iTypeOfRead) = 0;
	virtual BOOL STDMETHODCALLTYPE ReadImageFromMemory(BYTE* pBuf, int iSize, ETypeOfReadImage iTypeOfRead) = 0;
	virtual void STDMETHODCALLTYPE ReleaseImage() = 0;
	virtual BOOL STDMETHODCALLTYPE IsLoaded() = 0;
	virtual int STDMETHODCALLTYPE GetWidth() = 0;
	virtual int STDMETHODCALLTYPE GetHeight() = 0;
	virtual int STDMETHODCALLTYPE GetMCUWidth() = 0;
	virtual int STDMETHODCALLTYPE GetMCUHeight() = 0;
	virtual BOOL STDMETHODCALLTYPE Reconstruct(int iPasses, PFNPROGRESS pfnProgress) = 0;
	virtual BOOL STDMETHODCALLTYPE GetBitmap(BYTE* pScan0, int iStride, BOOL bWholeMCUs) = 0;
	virtual int STDMETHODCALLTYPE GetImageSize() = 0;
	virtual BOOL STDMETHODCALLTYPE Save(WCHAR* pwsFileName, save_params* pParams) = 0;
	virtual BOOL STDMETHODCALLTYPE SaveToMemory(BYTE* pBuf, int iSize) = 0;
	virtual BOOL STDMETHODCALLTYPE CreatePreview(WCHAR* pwsFileName, BYTE* pBits, int iThumbWidth,
		int iThumbHeight, int iStride, BYTE iR, BYTE iG, BYTE iB,
		int* piImageWidth, int* piImageHeight, int iExtFlags) = 0;

	virtual BOOL STDMETHODCALLTYPE Crop(int iLeft, int iTop, int iRight, int iBottom) = 0;
	virtual BOOL STDMETHODCALLTYPE FlipH(BOOL bPreservePartial) = 0;
	virtual BOOL STDMETHODCALLTYPE FlipV(BOOL bPreservePartial) = 0;
	virtual BOOL STDMETHODCALLTYPE RotateLeft(BOOL bPreservePartial) = 0;
	virtual BOOL STDMETHODCALLTYPE RotateRight(BOOL bPreservePartial) = 0;
	virtual BOOL STDMETHODCALLTYPE ApplyBitmapChanges(BYTE** ppScan, int iStride, BOOL bForceRecompress, recompress_statistics * pStatistics) = 0;
	virtual BOOL STDMETHODCALLTYPE ConvertGrayscale() = 0;
	virtual BOOL STDMETHODCALLTYPE CanConvertGrayscale() = 0;
	virtual BOOL STDMETHODCALLTYPE AdjustLevels(int iYShift, int iCbShift, int iCrShift) = 0;
	virtual BOOL STDMETHODCALLTYPE CanAdjustLevels() = 0;
	virtual void STDMETHODCALLTYPE SetUndoStore(IUndoStore* pUndoStore) = 0;
	virtual BOOL STDMETHODCALLTYPE Undo() = 0;

	virtual WCHAR* STDMETHODCALLTYPE Format(WCHAR* pwsFormat, BOOL bUseFileTime, LCID iLocale, WCHAR* pwsDateFormat, WCHAR* pwsTimeFormat, BOOL bFractionsOfSeconds) = 0;
	virtual USHORT STDMETHODCALLTYPE GetOrientation() = 0;
	virtual void STDMETHODCALLTYPE SetOrientation(USHORT iOrientation) = 0;
	virtual thumbnail_info STDMETHODCALLTYPE GetThumbnailInfo() = 0;
	virtual BOOL STDMETHODCALLTYPE IsGrayscale() = 0;

	virtual void STDMETHODCALLTYPE GetCoefficients(INT16** ppCoefs, int* iSize) = 0;
	virtual BOOL STDMETHODCALLTYPE SetCoefficients(INT16* pCoefs, int iSize) = 0;
};

#endif //_BJPEG_H_
